#!/usr/bin/env node

const debug = require("debug")("ldi");

const program = require("commander");
const fs = require("fs");
const path = require("path");
const spawnSync = require("child_process").spawnSync;
const git = require("git-state");
const gitu = require("git-utils");
const gitrv = require("git-rev-sync");
const color = require("colors");

const ldiModules = [
  "util",
  "param",
  "account",
  "common",
  "wallet",
  "payment-bni",
  "loan",
  "investment",
  "report",
  "jsreport"
];

program
  .command("status")
  .description("git status all modules")
  .action(() => {
    gitStatus();
  });

program
  .command("fetch")
  .description("git fetch all modules")
  .action(async () => {
    await gitFetch();
    await gitStatus();
  });

program.command("kube [arguments...]").description("kubernetes");

program.on("command:kube", () => {
  const argv = process.argv;
  argv[1] = path.join(__dirname, "kube.js");
  argv.splice(2, 1);
  debug(argv);
  spawnSync(argv[0], argv.slice(1), {
    stdio: "inherit"
  });
});

program.on("command:*", () => {
  program.help();
  process.exit(1);
});

if (process.argv.length <= 2) {
  program.help();
  process.exit(1);
}

program.parse(process.argv);

function getBase() {
  let base;
  if (git.isGitSync(".")) {
    base = path.resolve("..");
  } else {
    base = path.resolve(".");
  }
  for (const mod of ldiModules) {
    const fn = path.join(base, mod);
    if (!fs.existsSync(fn)) {
      console.log(`Module ${mod} not found in ${fn}`);
      process.exit(1);
    }
  }
  return base;
}

async function gitStatus() {
  const base = getBase();
  for (const mod of ldiModules) {
    const fn = path.join(base, mod);
    const gst = git.checkSync(fn);
    const repo = gitu.open(fn);
    console.log(
      `${color.blue((mod + ":              ").slice(0, 13))} ${gitrv.short(
        fn
      )}  ${gst.branch}`
    );
    let clean = true;
    const abc = repo.getAheadBehindCount();
    if (abc.ahead !== 0) {
      clean = false;
      console.log(`   Ahead:     ${abc.ahead}`);
    }
    if (abc.behind !== 0) {
      clean = false;
      console.log(color.red(`   Behind:    ${abc.behind}`));
    }
    if (gst.dirty !== 0) {
      clean = false;
      console.log(color.red(`   Dirty:     ${gst.dirty}`));
    }
    if (gst.untracked !== 0) {
      clean = false;
      console.log(`   Untracked: ${gst.untracked}`);
    }
    if (clean) {
      console.log(color.green(`   CLEAN`));
    }
  }
}

async function gitFetch() {
  const base = getBase();
  for (const mod of ldiModules) {
    const fn = path.join(base, mod);
    console.log(`${color.blue(mod)}:`);
    spawnSync("git", ["fetch", "--all"], {
      cwd: fn,
      stdio: "inherit"
    });
  }
}
